﻿Imports System.IO

Namespace Sonora.Console.Framework

    Public Class SonoraOutput

        Private ReadOnly _settings As SonoraSettings

        Public Sub New(Optional ByVal settings As SonoraSettings = Nothing)
            _settings = If(settings, New SonoraSettings)
        End Sub

        Public Function print(ByVal text As Object) As SonoraOutput
            If _settings.AutoClear Then
                clr()
            End If

            printinternal(_settings.Buffer, "{0}", text)

            If _settings.AppendNewLine Then
                _settings.Buffer.Write(vbCrLf)
            End If

            Return Me
        End Function

        Public Function print(ByVal format As String, ByVal ParamArray args() As Object) As SonoraOutput
            If _settings.AutoClear Then
                clr()
            End If

            printinternal(_settings.Buffer, format, args)

            If _settings.AppendNewLine Then
                _settings.Buffer.Write(vbCrLf)
            End If

            Return Me
        End Function

        Public Sub file(Optional ByVal filename As String = "")
            file(filename, "{0}", _settings.Buffer)
        End Sub

        Public Sub file(ByVal filename As String, ByVal text As Object)
            file(filename, "{0}", text)
        End Sub

        Public Sub file(ByVal filename As String, ByVal format As String, ByVal ParamArray args() As Object)
            If String.IsNullOrWhiteSpace(filename) Then
                filename = _settings.DefaultFileName
            End If
            Dim fullpath = Path.Combine(_settings.OutputFolder, filename)
            If Not Path.HasExtension(fullpath) Then
                fullpath = Path.ChangeExtension(fullpath, ".txt")
            End If

            Dim dirname = Path.GetDirectoryName(fullpath)
            If Not String.IsNullOrWhiteSpace(dirname) Then
                Directory.CreateDirectory(dirname)
            End If

            Dim writer = New StreamWriter(fullpath)
            printinternal(writer, format, args)
            writer.Close()

            If _settings.AppendNewLine Then
                _settings.Buffer.Write(vbCrLf)
            End If

            ' Out to screen too
            'print("Saved to file: {0}.", DateTime.Now.ToString("HH:mm:ss"));
        End Sub

        Public Sub clr()
            _settings.Buffer.GetStringBuilder().Clear()
        End Sub

        Private Sub printinternal(ByVal writer As TextWriter, ByVal format As String, ByVal ParamArray args() As Object)
            writer.Write(format, args)
        End Sub
    End Class

End Namespace